import { css, LitElement, property, svg } from "lit-element";

const NODE_SIZE = 30;
const SPACING = 10;

export class HatGraphNode extends LitElement {
  @property() iconPath?: string;
  @property({ reflect: true, type: Boolean }) marked?: boolean;
  @property({ reflect: true, type: Boolean }) selected?: boolean;
  @property({ reflect: true, type: Boolean }) disabled?: boolean;
  @property({ reflect: true, type: Boolean }) graphstart?: boolean;
  @property({ reflect: true, type: Boolean }) nofocus?: boolean;

  connectedCallback() {
    super.connectedCallback();
    if (!this.hasAttribute("tabindex") && !this.nofocus)
      this.setAttribute("tabindex", "0");
  }

  updated() {
    const svg = this.shadowRoot.querySelector("svg");
    const bbox = svg.getBBox();
    const extra_height = this.graphstart ? 2 : 1;
    const extra_width = SPACING;
    svg.setAttribute("width", `${bbox.width + extra_width}px`);
    svg.setAttribute("height", `${bbox.height + extra_height}px`);
    svg.setAttribute(
      "viewBox",
      `${Math.ceil(bbox.x - extra_width / 2)}
      ${Math.ceil(bbox.y - extra_height / 2)}
      ${bbox.width + extra_width}
      ${bbox.height + extra_height}`
    );
  }

  render() {
    return svg`
    <svg
    >
      ${
        this.graphstart
          ? ``
          : svg`
          <path
            class="connector"
            d="
              M 0 ${-SPACING - NODE_SIZE / 2}
              L 0 0
            "
            line-caps="round"
          />
          `
      }
    <g class="node">
      <circle
        cx="0"
        cy="0"
        r="${NODE_SIZE / 2}"
      />
      <g
        style="pointer-events: none"
        transform="translate(${-12} ${-12})"
      >
        ${this.iconPath ? svg`<path d="${this.iconPath}"/>` : ""}
      </g>
    </g>
      </svg>
    `;
  }

  static get styles() {
    return css`
      :host {
        display: flex;
        flex-direction: column;
        --stroke-clr: var(--stroke-color, rgb(3, 169, 244));
        --selected-clr: var(--selected-color, rgb(255, 152, 0));
        --marked-clr: var(--marked-color, springgreen);
        --hover-clr: var(--hover-color, red);
        --disabled-clr: var(--disabled-color, gray);
        --default-trigger-color: 3, 169, 244;
        --rgb-trigger-color: var(--trigger-color, var(--default-trigger-color));
      }
      :host([marked]) {
        --stroke-clr: var(--marked-clr);
      }
      :host([selected]),
      :host(:focus) {
        --stroke-clr: var(--selected-clr);
        outline: none;
      }
      :host(:hover) circle {
        --stroke-clr: var(--hover-clr);
      }
      :host([disabled]) circle {
        stroke: var(--disabled-clr);
      }
      :host-context([disabled]) {
        --stroke-clr: var(--disabled-clr);
      }

      :host([nofocus]):host-context([selected]),
      :host([nofocus]):host-context(:focus) {
        --stroke-clr: var(--selected-clr);
      }

      circle,
      path.connector {
        stroke: var(--stroke-clr);
        stroke-width: 2;
        fill: none;
      }
      circle {
        fill: white;
        stroke: var(--circle-clr, var(--stroke-clr));
      }

      :host(.triggered) svg {
        overflow: visible;
      }
      :host(.triggered) circle {
        animation: glow 10s;
      }
      @keyframes glow {
        0% {
          filter: drop-shadow(0px 0px 5px rgba(var(--rgb-trigger-color), 0));
        }
        10% {
          filter: drop-shadow(0px 0px 10px rgba(var(--rgb-trigger-color), 1));
        }
        100% {
          filter: drop-shadow(0px 0px 5px rgba(var(--rgb-trigger-color), 0));
        }
      }
    `;
  }
}

customElements.define("hat-graph-node", HatGraphNode);
