import { css, html, LitElement, property, svg } from "lit-element";
import { classMap } from "lit-html/directives/class-map";

const BRANCH_HEIGHT = 30;
const BRANCH_CURVATURE = 25;

const mark_converter = {
  fromAttribute: (value, type) => value.split(",").map((v) => parseInt(v)),
  toAttribute: (value, type) =>
    value instanceof Array ? value.join(",") : `${value}`,
};

export class HatGraph extends LitElement {
  @property() _num_items = 0;

  @property({ reflect: true, type: Boolean }) branching?: boolean;
  @property({ reflect: true, converter: mark_converter }) mark_start?: number[];
  @property({ reflect: true, converter: mark_converter }) mark_end?: number[];
  @property({ reflect: true, type: Boolean }) disabled?: boolean;
  @property({ reflect: true, type: Boolean }) selected?: boolean;

  async updateChildren() {
    this._num_items = this.children.length;
  }

  render() {
    let branches = [];
    let total_width = 0;
    let max_height = 0;
    let min_height = Number.POSITIVE_INFINITY;
    if (this.branching) {
      for (const c of Array.from(this.children)) {
        if (c.slot === "head") continue;
        const rect = c.getBoundingClientRect();
        branches.push({
          x: rect.width / 2 + total_width,
          height: rect.height,
          start: c.getAttribute("graphStart") != null,
          end: c.getAttribute("graphEnd") != null,
        });
        total_width += rect.width;
        max_height = Math.max(max_height, rect.height);
        min_height = Math.min(min_height, rect.height);
      }
    }

    return html`
      <slot name="head" @slotchange=${this.updateChildren}> </slot>
      ${this.branching
        ? svg`
            <svg
              id="top"
              width="${total_width}"
              height="${BRANCH_HEIGHT}"
            >
              ${branches.map((branch, i) => {
                if (branch.start) return "";
                return svg`
                  <path
                    class="${classMap({
                      line: true,
                      marked: this.mark_start?.includes(i),
                    })}"
                    id="${this.mark_start?.includes(i) ? "mark-start" : ""}"
                    index=${i}
                    d="
                      M ${total_width / 2} 0
                      C ${total_width / 2} ${BRANCH_CURVATURE}
                        ${branch.x} ${BRANCH_HEIGHT - BRANCH_CURVATURE}
                        ${branch.x} ${BRANCH_HEIGHT}
                      "/>
                `;
              })}
              <use xlink:href="#mark-start" />
            </svg>
          `
        : ""}
      <div id="branches">
        ${this.branching
          ? svg`
              <svg
                id="lines"
                width="${total_width}"
                height="${max_height}"
              >
                ${branches.map((branch, i) => {
                  if (branch.end) return "";
                  return svg`
                    <path
                      class="${classMap({
                        line: true,
                        marked: this.mark_end?.includes(i),
                      })}"
                      index=${i}
                      d="
                        M ${branch.x} ${branch.height}
                        l 0 ${max_height - branch.height}
                        "/>
                  `;
                })}
              </svg>
            `
          : ""}
        <slot @slotchange=${this.updateChildren}></slot>
      </div>

      ${this.branching
        ? svg`
            <svg
              id="bottom"
              width="${total_width}"
              height="${BRANCH_HEIGHT}"
            >
              ${branches.map((branch, i) => {
                if (branch.end) return "";
                return svg`
                  <path
                    class="${classMap({
                      line: true,
                      marked: this.mark_end?.includes(i),
                    })}"
                    id="${this.mark_end?.includes(i) ? "mark-end" : ""}"
                    index=${i}
                    d="
                      M ${branch.x} 0
                      C ${branch.x} ${BRANCH_CURVATURE}
                        ${total_width / 2} ${BRANCH_HEIGHT - BRANCH_CURVATURE}
                        ${total_width / 2} ${BRANCH_HEIGHT}
                      "/>
                `;
              })}
              <use xlink:href="#mark-end" />
            </svg>
          `
        : ""}
    `;
  }

  static get styles() {
    return css`
      :host {
        position: relative;
        display: flex;
        flex-direction: column;
        align-items: center;
        --stroke-clr: var(--stroke-color, rgb(3, 169, 244));
        --selected-clr: var(--selected-color, rgb(255, 152, 0));
        --marked-clr: var(--marked-color, springgreen);
        --disabled-clr: var(--disabled-color, gray);
      }
      :host(:focus) {
        outline: none;
      }
      #branches {
        position: relative;
        display: flex;
        flex-direction: column;
        align-items: center;
      }
      :host([branching]) #branches {
        flex-direction: row;
        align-items: start;
      }

      :host([branching]) ::slotted([slot="head"]) {
        margin-bottom: ${-BRANCH_HEIGHT / 2}px;
        z-index: 1;
      }

      #lines {
        position: absolute;
        z-index: -1;
      }

      path.line {
        stroke: var(--stroke-clr);
        stroke-width: 2;
        fill: none;
      }
      path.line.marked {
        stroke: var(--marked-clr);
      }
      :host([disabled]) path.line {
        stroke: var(--disabled-clr);
      }
      :host([selected]) #top path.line {
        stroke: var(--selected-clr);
      }
      :host(:focus) #top path.line {
        stroke: var(--selected-clr);
      }
    `;
  }
}

customElements.define("hat-graph", HatGraph);
