import "./hat-graph";
import "./hat-graph-node";
import { HatGraphNode } from "./hat-graph-node";
import { classMap } from "lit-html/directives/class-map";

import {
  mdiCallSplit,
  mdiAbTesting,
  mdiCheck,
  mdiClose,
  mdiChevronRight,
  mdiExclamation,
  mdiTimerOutline,
  mdiTrafficLight,
  mdiRefresh,
  mdiArrowUp,
  mdiCodeJson,
  mdiCheckBoxOutline,
  mdiCheckboxBlankOutline,
  mdiAsterisk,
} from "@mdi/js";
import { HatGraph } from "./hat-graph";
import { html } from "lit-element";
import { render } from "lit-html";
import {
  AutomationActionConfig,
  AutomationConfig,
  AutomationTrace,
} from "./types";

const OPTIONS = [
  "condition",
  "delay",
  "device_id",
  "event",
  "scene",
  "service",
  "wait_template",
  "repeat",
  "choose",
];

const ICONS = {
  new: mdiAsterisk,
  service: mdiChevronRight,
  condition: mdiAbTesting,
  TRUE: mdiCheck,
  FALSE: mdiClose,
  delay: mdiTimerOutline,
  wait_template: mdiTrafficLight,
  event: mdiExclamation,
  repeat: mdiRefresh,
  repeatReturn: mdiArrowUp,
  choose: mdiCallSplit,
  chooseChoice: mdiCheckBoxOutline,
  chooseDefault: mdiCheckboxBlankOutline,
  YAML: mdiCodeJson,
  trigger: mdiAsterisk,
};

const SPECIAL_NODES = {
  condition: makeConditionNode,
  choose: makeChooseNode,
  repeat: makeRepeatNode,
};

function makeConditionNode(
  config: AutomationActionConfig,
  path: string,
  trace?: AutomationTrace
) {
  const graph = document.createElement("hat-graph") as HatGraph;
  graph.branching = true;
  graph.setAttribute("tabindex", "0");

  let marked_branch;
  if (trace && path in trace) {
    marked_branch = trace[path].result ? "0" : "1";
    graph.setAttribute("mark_start", marked_branch);
    graph.setAttribute("mark_end", marked_branch);
  }

  graph.addEventListener("focus", () => {
    graph.dispatchEvent(
      new CustomEvent("node-selected", {
        detail: { path, config, node: graph },
        bubbles: true,
      })
    );
  });

  render(
    html`
      <hat-graph-node
        slot="head"
        .iconPath=${mdiAbTesting}
        ?marked=${trace && path in trace}
        nofocus
      >
      </hat-graph-node>

      <hat-graph-node
        .iconPath=${mdiCheck}
        nofocus
        ?marked=${marked_branch === "0"}
      ></hat-graph-node>
      <hat-graph-node
        .iconPath=${mdiClose}
        graphEnd
        nofocus
        ?marked=${marked_branch === "1"}
      ></hat-graph-node>
    `,
    graph
  );
  return graph;
}

function makeChooseNode(
  config: AutomationActionConfig,
  path: string,
  trace?: AutomationTrace
) {
  const graph = document.createElement("hat-graph") as HatGraph;
  graph.branching = true;
  graph.setAttribute("tabindex", "0");

  graph.addEventListener("focus", () => {
    graph.dispatchEvent(
      new CustomEvent("node-selected", {
        detail: { path, config, node: graph },
        bubbles: true,
      })
    );
  });

  if (trace && path in trace) {
    for (const [i, _] of Object.entries(config.choose)) {
      if (`${path}/choose/${i}` in trace) {
        graph.setAttribute("mark_start", `${i}`);
        console.log(config.choose[i], config.choose[i].length);
        if (
          `${path}/choose/${i}/sequence/${
            config.choose[i].sequence.length - 1
          }` in trace
        )
          graph.setAttribute("mark_end", `${i}`);
        break;
      }
    }
    if (`${path}/default` in trace) {
      graph.setAttribute("mark_start", `${config.choose.length}`);
      if (`${path}/default/${config.default.length - 1}` in trace)
        graph.setAttribute("mark_end", `${config.choose.length}`);
    }
  }

  render(
    html`
      <hat-graph-node
        slot="head"
        .iconPath="${mdiCallSplit}"
        nofocus
        ?marked=${trace && path in trace}
      >
      </hat-graph-node>
      ${config.choose?.map((branch, i) => {
        const pth = `${path}/choose/${i}`;
        const head = document.createElement("hat-graph-node") as HatGraphNode;
        head.iconPath = mdiCheckBoxOutline;
        head.setAttribute("nofocus", "");
        if (trace && pth in trace) {
          head.setAttribute("marked", "");
        }
        return makeGraph(branch.sequence, `${pth}/sequence`, head, trace);
      })}
      ${(() => {
        const pth = `${path}/default`;
        const head = document.createElement("hat-graph-node") as HatGraphNode;
        head.iconPath = mdiCheckboxBlankOutline;
        head.setAttribute("nofocus", "");
        if (trace && pth in trace) {
          head.setAttribute("marked", "");
        }
        return makeGraph(config.default, pth, head, trace);
      })()}
    `,
    graph
  );
  return graph;
}

function makeRepeatNode(
  config: AutomationActionConfig,
  path: string,
  trace?: AutomationTrace
) {
  const graph = document.createElement("hat-graph") as HatGraph;
  graph.branching = true;
  graph.setAttribute("tabindex", "0");

  if (trace && path in trace) {
    const marks = trace[path].repeats ? "0,1" : "1";
    graph.setAttribute("mark_start", marks);
    if (`${path}/sequence/${config.repeat.sequence.length - 1}` in trace)
      graph.setAttribute("mark_end", marks);
  }

  graph.addEventListener("focus", () => {
    graph.dispatchEvent(
      new CustomEvent("node-selected", {
        detail: { path, config, node: graph },
        bubbles: true,
      })
    );
  });

  render(
    html`
      <hat-graph-node
        slot="head"
        .iconPath=${mdiRefresh}
        nofocus
        ?marked=${trace && path in trace}
      ></hat-graph-node>
      <hat-graph-node
        .iconPath=${mdiArrowUp}
        nofocus
        ?marked=${trace && trace[path]?.repeats}
      ></hat-graph-node>
      ${makeGraph(config.repeat.sequence, `${path}/sequence`, undefined, trace)}
    `,
    graph
  );
  return graph;
}

function makeNode(
  config: AutomationActionConfig,
  path: string,
  trace?: AutomationTrace
) {
  const type = OPTIONS.find((key) => key in config) || "yaml";

  if (type in SPECIAL_NODES) {
    return SPECIAL_NODES[type](config, path, trace);
  }

  const node = document.createElement("hat-graph-node") as HatGraphNode;

  node.iconPath = ICONS[type];
  if (trace && path in trace) {
    node.setAttribute("marked", "");
  }

  node.addEventListener("focus", (ev) => {
    node.dispatchEvent(
      new CustomEvent("node-selected", {
        detail: { path, config, node },
        bubbles: true,
      })
    );
  });

  return node;
}

export function makeGraph(
  nodes: AutomationActionConfig[],
  path: string,
  head = undefined,
  trace?: AutomationTrace
) {
  const graph = document.createElement("hat-graph") as HatGraph;

  if (head) {
    head.slot = "head";
    graph.appendChild(head);
  }

  for (const [i, nodeConfig] of nodes.entries()) {
    const node = makeNode(nodeConfig, `${path}/${i}`, trace);

    node.addEventListener("update-node", (ev) => {
      ev.stopPropagation();

      const config = [...nodes];
      config[i] = ev.detail.config;

      graph.dispatchEvent(
        new CustomEvent("update-node", { detail: { config }, bubbles: true })
      );
    });

    node.addEventListener("delete-node", (ev) => {
      ev.stopPropagation();

      const config = [...nodes];
      config.splice(i, 1);

      graph.dispatchEvent(
        new CustomEvent("update-node", { detail: { config }, bubbles: true })
      );
    });

    node.addEventListener("place-node", (ev) => {
      ev.stopPropagation();

      const config = [...nodes];
      config.splice(i, 0, ev.detail.config);
      console.log(config);

      graph.dispatchEvent(
        new CustomEvent("update-node", { detail: { config }, bubbles: true })
      );
    });

    graph.appendChild(node);
  }

  return graph;
}

export function automationGraph(
  automation: AutomationConfig,
  trace: AutomationTrace = {}
) {
  const graph = document.createElement("hat-graph");

  const selected = (config, path: string) => {
    return (ev) => {
      const node = ev.target;
      graph.dispatchEvent(
        new CustomEvent("node-selected", {
          detail: { path, config, node },
          bubbles: true,
        })
      );
    };
  };

  let marked_trigger = [];
  // This part finds which trigger trace to colorize
  for (const [i, trigger] of Object.entries(automation.trigger)) {
    if (trace && `trigger/${i}` in trace) {
      marked_trigger.push(i);
    }

    // This would be a real ws event listener, I guess.
    // Inspired by the animations when tags are scanned.
    // Could be added to wait-for-trigger nodes and stuff too
    document.addEventListener("event-triggered", (ev: CustomEvent) => {
      if (ev.detail.trigger === trigger) {
        const el = graph.querySelector(
          `#trigger hat-graph-node:nth-of-type(${parseInt(i) + 1})`
        ) as any;
        if (el) {
          if (el._timeout) {
            clearTimeout(el._timeout);
            el._timeout = undefined;
            el.classList.remove("triggered");
            void el.offsetWidth;
          }
          el.classList.add("triggered");

          el._timeout = window.setTimeout(() => {
            el.classList.remove("triggered");
            el._timeout = undefined;
          }, 10000);
        }
      }
    });
  }

  for (const [i, condition] of Object.entries(automation.condition)) {
    document.addEventListener("states-changed", (ev: CustomEvent) => {
      const el = graph.querySelector(
        `#condition hat-graph-node:nth-of-type(${parseInt(i) + 1})`
      ) as any;
      console.log(ev.detail);
      if (ev.detail.state[condition]) {
        el.iconPath = mdiCheck;
      } else {
        el.iconPath = mdiClose;
      }
    });
  }

  render(
    html`
      <hat-graph branching mark_end=${marked_trigger.join(",")} id="trigger">
        ${automation.trigger.map((trigger, i) => {
          const path = `trigger/${i}`;
          return html`
            <hat-graph-node
              .iconPath=${mdiAsterisk}
              graphStart
              @focus=${selected(trigger, path)}
              ?marked=${trace && path in trace}
            ></hat-graph-node>
          `;
        })}
      </hat-graph>

      <hat-graph id="condition">
        ${automation.condition.map((condition, i) => {
          const path = `condition/${i}`;
          return html`
            <hat-graph-node
              .iconPath=${mdiCheck}
              @focus=${selected(condition, path)}
              ?marked=${trace && path in trace}
            ></hat-graph-node>
          `;
        })}
      </hat-graph>

      <hat-graph>
        ${automation.action.map((action, i) => {
          return makeNode(action, `action/${i}`, trace);
        })}
      </hat-graph>
    `,
    graph
  );
  return graph;
}
